package com.landray.kmss.sys.attachment.service.spring;

import java.awt.image.BufferedImage;
import javax.imageio.ImageIO;
import com.landray.kmss.util.StringUtil;
import java.util.HashMap;
import com.landray.kmss.util.ClassUtils;
import java.io.InputStream;
import java.io.ByteArrayInputStream;
import java.util.List;
import com.landray.kmss.common.model.IBaseModel;
import java.util.Iterator;
import net.sf.json.JSONObject;
import java.util.Map;
import java.io.OutputStream;
import com.landray.kmss.sys.attachment.util.ImageCropUtil;
import org.apache.commons.io.FilenameUtils;
import java.io.ByteArrayOutputStream;
import com.landray.kmss.sys.log.util.oper.UserOperContentHelper;
import com.landray.kmss.sys.log.util.UserOperHelper;
import com.landray.kmss.sys.attachment.model.SysAttMain;
import com.landray.kmss.util.UserUtil;
import com.landray.kmss.sys.attachment.forms.SysAttImageCropForm;
import net.sf.json.JSONArray;
import com.landray.kmss.common.forms.IExtendForm;
import com.landray.kmss.sys.attachment.service.ISysAttMainCoreInnerService;
import com.landray.kmss.sys.attachment.service.ISysAttImageCropService;
import com.landray.kmss.common.service.BaseServiceImp;

public class SysAttImageCropServiceImp extends BaseServiceImp implements ISysAttImageCropService
{
    protected ISysAttMainCoreInnerService sysAttMainService;
    
    public void setSysAttMainService(final ISysAttMainCoreInnerService sysAttMainService) {
        this.sysAttMainService = sysAttMainService;
    }
    
    public JSONArray addCrop(final IExtendForm form) throws Exception {
        final SysAttImageCropForm cropForm = (SysAttImageCropForm)form;
        final String fdModelName = cropForm.getFdModelName();
        if ("com.landray.kmss.sys.zone.model.SysZonePersonInfo".equals(fdModelName) && !cropForm.getFdModelId().equals(UserUtil.getKMSSUser().getUserId()) && !UserUtil.getKMSSUser().isAdmin()) {
            return null;
        }
        final String fdAttId = cropForm.getFdCropId();
        final SysAttMain att = this.getAtt(fdAttId);
        JSONArray obj = null;
        if (att != null) {
            obj = this.cropAndCompress(cropForm, att);
            this.sysAttMainService.delete(fdAttId);
            if (UserOperHelper.allowLogOper("addCrop", SysAttMain.class.getName())) {
                UserOperContentHelper.putDelete(fdAttId, att.getFdFileName(), SysAttMain.class.getName());
            }
        }
        return obj;
    }
    
    private JSONArray cropAndCompress(final SysAttImageCropForm cropForm, final SysAttMain attMain) throws Exception {
        final ByteArrayOutputStream out = new ByteArrayOutputStream();
        ImageCropUtil.cropImage(attMain.getInputStream(), (OutputStream)out, (int)cropForm.getFdStartX(), (int)cropForm.getFdStartY(), (int)cropForm.getFdCropWidth(), (int)cropForm.getFdCropHeight(), FilenameUtils.getExtension(attMain.getFdFileName()));
        return this.compressImage(cropForm, attMain, out);
    }
    
    private JSONArray compressImage(final SysAttImageCropForm cropForm, final SysAttMain attMain, final ByteArrayOutputStream out) throws Exception {
        final Map<String, Integer[]> map = this.getCompressWHMap(cropForm, attMain.getFdKey());
        final String fileName = attMain.getFdFileName();
        final byte[] bytes = out.toByteArray();
        final JSONArray array = new JSONArray();
        String dUrl = null;
        for (final Map.Entry<String, Integer[]> entry : map.entrySet()) {
            final String fdModelName = cropForm.getFdModelName();
            if ("com.landray.kmss.sys.zone.model.SysZonePersonInfo".equals(fdModelName)) {
                this.delLastCrop(cropForm, entry.getKey());
            }
            dUrl = this.compressByKey(cropForm, entry, bytes, fileName);
            final JSONObject json = new JSONObject();
            json.put((Object)"key", (Object)entry.getKey());
            json.put((Object)"value", (Object)dUrl.substring(dUrl.indexOf("fdId=") + 5));
            array.add((Object)json);
            if (UserOperHelper.allowLogOper("addCrop", SysAttMain.class.getName())) {
                UserOperContentHelper.putAdd(dUrl.substring(dUrl.indexOf("fdId=") + 5), fileName, SysAttMain.class.getName());
            }
        }
        return array;
    }
    
    private void delLastCrop(final SysAttImageCropForm cropForm, final String fdKey) throws Exception {
        final List<SysAttMain> attMains = this.sysAttMainService.findByModelKey(cropForm.getFdModelName(), cropForm.getFdModelId(), fdKey);
        for (final SysAttMain sysAttMain : attMains) {
            this.sysAttMainService.delete((IBaseModel)sysAttMain);
        }
    }
    
    private String compressByKey(final SysAttImageCropForm cropForm, final Map.Entry<String, Integer[]> entry, final byte[] bytes, final String fileName) throws Exception {
        final ByteArrayOutputStream out = new ByteArrayOutputStream();
        ImageCropUtil.compress((InputStream)new ByteArrayInputStream(bytes), (OutputStream)out, (int)entry.getValue()[0], (int)entry.getValue()[1], fileName);
        final IBaseModel model = ClassUtils.forName(cropForm.getFdModelName()).newInstance();
        model.setFdId(cropForm.getFdModelId());
        return this.sysAttMainService.addAttachment(model, (String)entry.getKey(), out.toByteArray(), fileName, "pic");
    }
    
    private Map<String, Integer[]> getCompressWHMap(final SysAttImageCropForm cropForm, final String fdKey) {
        final Map<String, Integer[]> map = new HashMap<String, Integer[]>();
        final String fdCropKeys = cropForm.getFdCropKeys();
        final String[] cropKeys = ImageCropUtil.CROP_KEYS;
        final boolean isAll = StringUtil.isNull(fdCropKeys);
        if (isAll || fdCropKeys.contains(cropKeys[0])) {
            map.put(String.valueOf(fdKey) + cropKeys[0], cropForm.getBigWH());
        }
        if (isAll || fdCropKeys.contains(cropKeys[1])) {
            map.put(String.valueOf(fdKey) + cropKeys[1], cropForm.getMediumWH());
        }
        if (isAll || fdCropKeys.contains(cropKeys[2])) {
            map.put(String.valueOf(fdKey) + cropKeys[2], cropForm.getSmallWH());
        }
        return map;
    }
    
    public void updateCancel(final String fdAttId) throws Exception {
        this.sysAttMainService.delete(fdAttId);
    }
    
    public JSONArray findByKeys(final String modelName, final String modelId, final String[] keys) throws Exception {
        final JSONArray obj = new JSONArray();
        for (int i = 0; i < keys.length; ++i) {
            this.buildData(modelName, modelId, keys[i], obj);
        }
        return obj;
    }
    
    private void buildData(final String modelName, final String modelId, final String key, final JSONArray arr) throws Exception {
        final List<SysAttMain> list = this.sysAttMainService.findByModelKey(modelName, modelId, key);
        if (!list.isEmpty()) {
            final JSONObject obj = new JSONObject();
            obj.accumulate("key", (Object)key);
            obj.accumulate("value", (Object)list.get(0).getFdId());
            arr.add((Object)obj);
        }
    }
    
    public JSONObject obtainImageInfo(final String fdAttId) throws Exception {
        final JSONObject json = new JSONObject();
        final SysAttMain att = this.getAtt(fdAttId);
        if (att != null) {
            final BufferedImage sourceImg = ImageIO.read(att.getInputStream());
            json.accumulate("width", sourceImg.getWidth());
            json.accumulate("height", sourceImg.getHeight());
        }
        return json;
    }
    
    private SysAttMain getAtt(final String fdAttId) throws Exception {
        final List<SysAttMain> list = this.sysAttMainService.findModelsByIds(new String[] { fdAttId });
        if (list.isEmpty()) {
            return null;
        }
        return list.get(0);
    }
}
